@@ -18,7 +18,7 @@ from crypten.common.rng import generate_random_ring_element
 from crypten.common.tensor_types import is_float_tensor
 from crypten.common.util import count_wraps
 from crypten.mpc.primitives import ArithmeticSharedTensor
-from test.multiprocess_test_case import MultiProcessTestCase, get_random_test_tensor
+from test.multiprocess_test_case import get_random_test_tensor, MultiProcessTestCase
 
 
 class TestArithmetic(MultiProcessTestCase):
